''' <summary>
''' Manages a single customer
''' </summary>
''' <remarks></remarks>
Public Class Customer
    Implements System.ComponentModel.INotifyPropertyChanged

#Region " Events"
    Public Event PropertyChanged(ByVal sender As Object, _
        ByVal e As System.ComponentModel.PropertyChangedEventArgs) _
        Implements System.ComponentModel.INotifyPropertyChanged.PropertyChanged
#End Region

#Region " Enums"
    Public Enum EntityStateEnum
        Unchanged
        Added
        Deleted
        Modified
    End Enum
#End Region

#Region " Properties"
    Private _EntityState As EntityStateEnum
    ''' <summary>
    ''' State of the customer object data
    ''' </summary>
    ''' <value></value>
    ''' <returns>Entity state</returns>
    ''' <remarks>This property should be in a business object base class</remarks>
    Public Property EntityState() As EntityStateEnum
        Get
            Return _EntityState
        End Get
        Set(ByVal value As EntityStateEnum)
            _EntityState = value
        End Set
    End Property

    Private _CustomerID As Int32
    ''' <summary>
    ''' ID of the Customer
    ''' </summary>
    ''' <value></value>
    ''' <returns>Customer ID</returns>
    ''' <remarks></remarks>
    Public Property CustomerID() As Int32
        Get
            Return _CustomerID
        End Get
        Private Set(ByVal value As Int32)
            _CustomerID = value
        End Set
    End Property

    Private _LastName As String
    ''' <summary>
    ''' Last name of the customer. 
    ''' </summary>
    ''' <value></value>
    ''' <returns>Last name</returns>
    ''' <remarks>This is a required field.</remarks>
    Public Property LastName() As String
        Get
            Return _LastName
        End Get
        Set(ByVal value As String)
            If String.IsNullOrEmpty(value) Then
                Throw New Exception("The Last Name cannot be empty")
            End If
            If value <> _LastName Then
                Me.DataStateChanged(EntityStateEnum.Modified, "LastName")
            End If
            _LastName = value
        End Set
    End Property

    Private _FirstName As String
    ''' <summary>
    ''' First name of the customer.
    ''' </summary>
    ''' <value></value>
    ''' <returns>First name</returns>
    ''' <remarks></remarks>
    Public Property FirstName() As String
        Get
            Return _FirstName
        End Get
        Set(ByVal value As String)
            If value <> _FirstName Then
                Me.DataStateChanged(EntityStateEnum.Modified, "FirstName")
            End If
            _FirstName = value
        End Set
    End Property

    Private _InvoiceViaEmail As Boolean
    Public Property InvoiceViaEmail() As Boolean
        Get
            Return _InvoiceViaEmail
        End Get
        Set(ByVal value As Boolean)
            If value <> _InvoiceViaEmail Then
                Me.DataStateChanged(EntityStateEnum.Modified, "InvoiceViaEmail")

                ' If this is on, turn off the other two
                If value = True Then
                    InvoiceViaFax = False
                    InvoiceViaPost = False
                End If
            End If
            _InvoiceViaEmail = value
        End Set
    End Property

    Private _InvoiceViaFAX As Boolean
    Public Property InvoiceViaFax() As Boolean
        Get
            Return _InvoiceViaFAX
        End Get
        Set(ByVal value As Boolean)
            If value <> _InvoiceViaFAX Then
                Me.DataStateChanged(EntityStateEnum.Modified, "InvoiceViaFAX")

                ' If this is on, turn off the other two
                If value = True Then
                    InvoiceViaEmail = False
                    InvoiceViaPost = False
                End If
            End If
            _InvoiceViaFAX = value
        End Set
    End Property

    Private _InvoiceViaPost As Boolean
    Public Property InvoiceViaPost() As Boolean
        Get
            Return _InvoiceViaPost
        End Get
        Set(ByVal value As Boolean)
            If value <> _InvoiceViaPost Then
                Me.DataStateChanged(EntityStateEnum.Modified, "InvoiceViaPost")

                ' If this is on, turn off the other two
                If value = True Then
                    InvoiceViaEmail = False
                    InvoiceViaFax = False
                End If
            End If
            _InvoiceViaPost = value
        End Set
    End Property

    ''' <summary>
    ''' Defines whether the state of this business object has been changed
    ''' </summary>
    ''' <value></value>
    ''' <returns>true if the state has changed; false if the state has not changed</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property IsDirty() As Boolean
        Get
            Return Me.EntityState <> EntityStateEnum.Unchanged
        End Get
    End Property
#End Region

#Region " Constructors"
    ''' <summary>
    ''' Constructor for this business object
    ''' </summary>
    ''' <remarks>This is set to provide to prevent direct creation
    ''' of this object. Instead, the object should be created using
    ''' the Create method</remarks>
    Private Sub New()
        ' Don't allow creation of this object directly
    End Sub
#End Region

#Region " Public Methods"

#Region " Create"
    ''' <summary>
    ''' Creates an instance of the class
    ''' </summary>
    ''' <param name="CustomerID">ID of the customer for which this instance is to be created</param>
    ''' <returns>Fully populated Customer instance</returns>
    ''' <remarks></remarks>
    Public Shared Function Create(ByVal CustomerID As Int32) As Customer
        ' Create the customer
        Dim oCustomer As New Customer

        ' Retrieve the data for this customer
        Dim dr As DataRow
        dr = oCustomer.Retrieve(CustomerID)

        ' Populate the business object from the data row
        oCustomer.Populate(dr)

        Return oCustomer
    End Function
#End Region

#End Region

#Region " Private Methods"

#Region " Populate"
    ''' <summary>
    ''' Populates the business object from the defined DataRow
    ''' </summary>
    ''' <param name="CustomerDataRow">DataRow containing data for this customer</param>
    ''' <remarks></remarks>
    Private Sub Populate(ByVal CustomerDataRow As DataRow)
        With CustomerDataRow
            Me.LastName = .Item("LastName").ToString
            Me.FirstName = .Item("FirstName").ToString
            Me.CustomerID = .Item("CustomerID").ToString

            ' This could be defined using an Enum
            Select Case .Item("InvoiceVia")
                Case 1
                    Me.InvoiceViaEmail = True
                Case 2
                    Me.InvoiceViaFax = True
                Case 3
                    Me.InvoiceViaPost = True
            End Select
        End With
    End Sub
#End Region

#Region " Retrieve"
    ''' <summary>
    ''' Retrieves the Customer data using the appropriate mechanism
    ''' </summary>
    ''' <param name="CustomerID">ID of the customer for which data is
    ''' to be retrieved</param>
    ''' <returns></returns>
    ''' <remarks>In a real application, this would call a data access component,
    ''' access a typed dataset or TableAdapter, or call a Web service.</remarks>
    Private Function Retrieve(ByVal CustomerID As Int32) As DataRow
        Dim dt As New DataTable

        ' Define the columns
        dt.Columns.Add("CustomerID")
        dt.Columns.Add("LastName")
        dt.Columns.Add("FirstName")
        dt.Columns.Add("InvoiceVia")

        Select Case CustomerID
            Case 1
                dt.Rows.Add(1, "Einstein", "Albert", 1)
            Case 2
                dt.Rows.Add(2, "Curie", "Marie", 2)
            Case 3
                dt.Rows.Add(3, "Brahe", "Tycho", 3)
            Case 4
                dt.Rows.Add(4, "Faraday", "Michael", 1)
        End Select

        Return dt.Rows(0)
    End Function
#End Region

#Region " DataStateChanged"
    ''' <summary>
    ''' Defines that a property is changed and that the system needs to be notified
    ''' </summary>
    ''' <param name="dataState">New state of the customer data</param>
    ''' <param name="propertyName">Name of the customer property that was changed</param>
    ''' <remarks>This code generates the PropertyChanged event</remarks>
    Private Sub DataStateChanged(ByVal dataState As EntityStateEnum, ByVal propertyName As String)
        ' Raise the event
        If Not String.IsNullOrEmpty(propertyName) Then
            RaiseEvent PropertyChanged(Me, _
               New System.ComponentModel.PropertyChangedEventArgs(propertyName))
        End If

        ' If the state is deleted, mark it as deleted
        If dataState = EntityStateEnum.Deleted Then
            Me.EntityState = dataState
        End If

        ' Only set other data states if the existing state is unchanged
        If Me.EntityState = EntityStateEnum.Unchanged Then
            Me.EntityState = dataState
        End If
    End Sub
#End Region

#End Region

End Class
